# Frequently Asked Questions

### I'm having trouble compiling `<project name here>`

First, make sure that you can compile that project natively on whatever platform you're attempting to compile it on.  Once you are assured of that, search around the internet to see if anyone else has run into issues cross-compiling that project for that platform.  In particular, most smaller projects should be just fine, but larger projects (and especially anything that does any kind of bootstrapping) may need some extra smarts smacked into their build system to support cross-compiling.  Finally, if you're still stuck, try reaching out for help on the `#bindeps2` channel in the JuliaLang slack.

### How do I use this to compile my Julia code?

This package does not compile Julia code; it compiles C/C++/Fortran dependencies.  Think about that time you wanted to use `IJulia` and you needed to download/install `libnettle`.  The purpose of this package is to make generated tarballs that can be downloaded/installed painlessly as possible.

### What is this I hear about the macOS SDK license agreement?

Apple restricts distribution and usage of the macOS SDK, a necessary component to build software for macOS targets.  Please read the [Apple and Xcode SDK agreement](https://images.apple.com/legal/sla/docs/xcode.pdf) for more information on the restrictions and legal terms you agree to when using the SDK to build software for Apple operating systems.  As usual, you should not take legal advice from FAQs on the internet, but in an effort to distill that large document down a bit, it is a breach of the license agreement to use the SDK to compile macOS binaries on a machine that is itself not a macOS machine.  Although this toolkit is designed to primarily run on Linux machines, it would not be breaking the license agreement to run this toolkit within a virtualized environment on a macOS machine, whereas it would be breaking the license agreement to run this toolkit on, for example, an Amazon AWS machine running Linux.  The QEMU runner (currently in testing, bug reports appreciated) implements the virtualization approach on macOS machines.  `BinaryBuilder.jl`, by default, will not automatically download or use the macOS SDK on non-apple operating systems, unless the `BINARYBUILDER_AUTOMATIC_APPLE` environment variable is set to `true`.

### Are there other environment variables I can use?

Yes, [take a look](environment_variables.md).

### Hey, this is cool, can I use this for my non-Julia related project?

Absolutely!  There's nothing Julia-specific about the binaries generated by the cross-compilers used by `BinaryBuilder.jl`.  Although the best interface for interacting with this software will always be the Julia interface defined within this package, you are free to use these software tools for other projects as well.  Note that the cross-compiler image is built within an enormous Docker image, [see this repository for more information](https://github.com/staticfloat/julia-docker/tree/master/crossbuild).  Further note the **macOS SDK license agreement** tidbit above.

### What platforms are supported?

At the time of writing, we support Linux (x86_64, i686, armv7l, aarch64, ppc64le), Windows (x86_64, i686) and macOS (x86_64).

### At line XXX, ABORTED (Operation not permitted)!

Some linux distributions have a bug in their `overlayfs` implementation that prevents us from mounting overlay filesystems within user namespaces.  See [this Ubuntu kernel bug report](https://bugs.launchpad.net/ubuntu/+source/linux/+bug/1531747) for a description of the situation and how Ubuntu has patched it in their kernels.  To work around this, you can launch `BinaryBuilder.jl` in "privileged container" mode.  Unfortunately, this involves running `sudo` every time you launch into a BinaryBuilder session, but on the other hand, this successfully works around the issue on distributions such as Arch linux.  To set "privileged container" mode, set the `BINARYBUILDER_RUNNER` environment variable to `privileged`.